# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

from datetime import datetime

import pyvista

import ogstools

project = "ogstools"
author = ogstools.__authors__
copyright = f"{datetime.now().year}, {author}"
version = release = ogstools.__version__

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autodoc.typehints",
    "sphinx.ext.viewcode",
    "sphinxarg.ext",
    "sphinxcontrib.apidoc",
    "sphinxcontrib.programoutput",
    "myst_nb",
    "sphinx_design",
    "sphinx_gallery.gen_gallery",
]

templates_path = ["_templates"]
exclude_patterns = [
    "_build",
    "Thumbs.db",
    ".DS_Store",
    "reference/modules.rst",
    "reference/ogstools.rst",
    "examples/**/README.rst",
    "examples/README.rst",
]

myst_enable_extensions = ["dollarmath", "colon_fence", "amsmath"]
myst_heading_anchors = 3
# myst_title_to_header = True

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "pydata_sphinx_theme"
html_static_path = ["_static"]
html_css_files = ["ogstools.css"]
html_context = {"default_mode": "light"}

html_theme_options = {
    "logo": {
        # "text": "OGSTools",
        "image_light": "ogstools.png",
        "image_dark": "ogstools-dark.png",
    },
    # "announcement": "<p style='font-weight: 400'>Some announcement</p>",
    "icon_links": [
        {
            "name": "GitLab",
            "url": "https://gitlab.opengeosys.org/ogs/tools/ogstools",
            "icon": "fa-brands fa-square-gitlab",
            "type": "fontawesome",
        }
    ],
}

nitpick_ignore_regex = [("py:class", r".*")]

show_authors = True

# sphinx-panels shouldn't add bootstrap css since the pydata-sphinx-theme
# already loads it
panels_add_bootstrap_css = False

### apidoc / autodoc settings ###
apidoc_module_dir = "../ogstools"
apidoc_output_dir = "reference"
apidoc_excluded_paths = ["../**/examples/**", "./**/tests/**"]
apidoc_separate_modules = True
apidoc_module_first = True
apidoc_extra_args = ["--force", "--implicit-namespaces"]

autoclass_content = "both"
autodoc_class_signature = "separated"
autodoc_member_order = "bysource"
autodoc_preserve_defaults = True
autodoc_typehints = "description"

### sphinx-gallery setup ###
# necessary when building the sphinx gallery
pyvista.BUILDING_GALLERY = True
pyvista.OFF_SCREEN = True

sphinx_gallery_conf = {
    "examples_dirs": ["examples"],
    "gallery_dirs": ["auto_examples"],
    "show_signature": False,
    "download_all_examples": False,
    "image_scrapers": ("matplotlib", "pyvista"),
    "matplotlib_animations": True,
}

# fe2vtu is optional
try:
    import ifm_contrib as ifm  # noqa: F401
except ImportError:
    print("Skipping FEFLOW-related functionality for the documentation!")
    exclude_patterns.extend(
        ["reference/ogstools.fe2vtu.*", "user-guide/fe2vtu*"]
    )
    apidoc_excluded_paths.append("../**/fe2vtu/**")
    sphinx_gallery_conf["ignore_pattern"] = r".*fe2vtu\.py"


# Suppress sphinx warning for multiple documents generated by sphinx-gallery
# May break myst-nb?
def setup(app):
    app.registry.source_suffix.pop(".ipynb", None)
